﻿using System;
using System.Collections.Generic;
using System.Text;



//there are actually two namespaces in this .cs (VATestPlugin, VATestWinampPlugin... see way down below) and each contain a class that would be considered a, 'plugin' (since each contain the necessary static functions to be a VoiceAttack plugin)

namespace VATestPlugin
{
    public class VoiceAttackPlugin
    {
        //NOTE THAT THIS IS A SAMPLE CLASS AND IS NOT FOR ANY MEANINGFUL USE.
        //You will want to gut this completely if you plan on using it... lol

        public static string VA_DisplayName()
        {
            return "My VoiceAttack Plugin - v1.5.9+";  //this is what you will want displayed in dropdowns as well as in the log file to indicate the name of your plugin
        }

        public static string VA_DisplayInfo()
        {
            return "My VoiceAttack Plugin\r\n\r\nThis is just a sample.\r\n\r\n2015 VoiceAttack";  //this is just extended info that you might want to give to the user.  note that you should format this up properly.
        }

        public static Guid VA_Id()
        {
            return new Guid("{C16BEBA9-2003-4b7f-BFA2-55B7C607925B}");  //this id must be generated by YOU... it must be unique so VoiceAttack can identify and use the plugin
        }

        //ref Dictionary<string, int> intValues, ref Dictionary<string, decimal> decimalValues, ref Dictionary<string, Boolean> booleanValues,

        public static void VA_Init1(ref Dictionary<string, object> state, ref Dictionary<string, Int16?> shortIntValues, ref Dictionary<string, string> textValues, ref Dictionary<string, int?> intValues, ref Dictionary<string, decimal?> decimalValues, ref Dictionary<string, Boolean?> booleanValues, ref Dictionary<string, DateTime?> dateTimeValues, ref Dictionary<string, object> extendedValues)
        {
            //this is where you can set up whatever session information that you want.  this will only be called once on voiceattack load, and it is called asynchronously.
            //the state parameter is a local copy of the state held on to by VoiceAttack.  In this case, the state will be a dictionary with zero items.  You can add as many items to hold on to as you want.
            //the conditions and textValues will also be empty.  You can add whatever you want to those lists and VoiceAttack will copy the values into its own lists

            shortIntValues.Add("initializedCondition1", 1);  //add some meaningless example short integers (used to be called, 'conditions')
            shortIntValues.Add("initializedCondition2", 2);

            textValues.Add("initializedText1", "This is 1");  //add some meaningless example text values
            textValues.Add("initializedText2", "This is 2");

            intValues.Add("initializedInt1", 55);  //add some meaningless example integer values
            intValues.Add("initializedInt2", 77);

            decimalValues.Add("initializedDecimal1", 44.2m);  //add some meaningless example decimal values
            decimalValues.Add("initializedDecimal2", -99.9m);

            booleanValues.Add("initializedBoolean1", true);  //add some meaningless example boolean values
            booleanValues.Add("initializedBoolean2", false);

            dateTimeValues.Add("initializedDate1", DateTime.Now);  //add some meaningless example date/time values
            dateTimeValues.Add("initializedDate2", new DateTime(2018, 1, 1));

            state.Add("new state value", 369);  //add whatever private state information you want to maintain
            state.Add("second new state value", "hello");
        }

        public static void VA_Exit1(ref Dictionary<string, object> state)
        {
            //this function gets called when VoiceAttack is closing (normally).  You would put your cleanup code in here, but be aware that your code must be robust enough to not absolutely depend on this function being called
            if (state.ContainsKey("myStateValue"))
            {
                //do some kind of file cleanup or whatever at this point
            }
        }

        public static void VA_Invoke1(String context, ref Dictionary<string, object> state, ref Dictionary<string, Int16?> shortIntValues, ref Dictionary<string, string> textValues, ref Dictionary<string, int?> intValues, ref Dictionary<string, decimal?> decimalValues, ref Dictionary<string, Boolean?> booleanValues, ref Dictionary<string, DateTime?> dateTimeValues, ref Dictionary<string, object> extendedValues)
        {
            //This function is where you will do all of your work.  When VoiceAttack encounters an, 'Execute External Plugin Function' action, the plugin indicated will be called.

            //What you are getting in this function as parameters :

            //context - a string that can be anything you want it to be.  this is passed in from the command action.  this was added to allow you to just pass a value into the plugin in a simple fashion (without having to set conditions/text values beforehand).  Convert the string to whatever type you need to.

            //three dictionaries which are all local copies :
            //state - all values from the state maintained by VoiceAttack for this plugin.  the state allows you to maintain kind of a, 'session' within VoiceAttack.  this value is not persisted to disk and will be erased on restart. other plugins do not have access to this state (private to the plugin)
            //shortIntValues - the short integer (used to be called, 'conditions') that were specified in the, 'Execute External' command action.
            //textValues - the text values that were specified in the, 'Execute External' command action.
            //intValues - the integer values that were specified in the, 'Execute External' command action.
            //decimalValues - the decimal values that were specified in the, 'Execute External' command action.
            //booleanValues - the boolean values that were specified in the, 'Execute External' command action.
            //dateTimeValues - the date/time values that were specified in the, 'Execute External' command action.
            //(you also get, 'extendedValues', but, that will be null because it is for future use)

            //the smallInt, text, integer, decimal, dateTime and boolean value dictionaries can be manipulated as follows :
            //items can be updated
            //items can be added to the dictionary
            //to indicate to VoiceAttack that you want to remove an item from the each dictionary, set its value to null (a process is already in place to remove items that are null for cleanup purposes, so, we can just use that)
            //any updated values will be reflected back to existing VoiceAttack internal values.
            //any added values will be added to the VoiceAttack internal values.

            //the state dictionary is the complete state.  you can manipulate it however you want, the whole thing will be copied back and replace what VoiceAttack is holding on to

            //if the, 'Execute External Plugin Function' command action has the, 'wait for return' flag set, VoiceAttack will wait until this function completes so that you may check condition values and
            //have VoiceAttack react accordingly.  otherwise, VoiceAttack fires and forgets and doesn't hang out for extra processing.


            //below is just some sample code showing how to work with dicitionaries...  in case you're not familiar with them.  just wouldn't feel complete unless i included it for some reason :)

            //here we test to see if the dictionary contains a certain key.  checking for the key first is important to avoid errors.  you can do this when setting a value and, 'TryGetValue' when you are retrieving values

            if (textValues.ContainsKey("myCSharpTestValue")) //was the text value passed in?
            {
                if (textValues["myCSharpTestValue"] == null) //if the value is null, set the value to, 'this is new'
                    textValues["myCSharpTestValue"] = "this is new";
                else
                    textValues["myCSharpTestValue"] = "hello " + new Random().Next(1, 6).ToString(); //if the value is not null, this is a subsequent call... just change up the value to nonsense.  this value will go back to VoiceAttack (perhaps to be read with TTS or whatever)
            }
            else //value was not passed in
                textValues.Add("myCSharpTestValue", "this is new");  //since the value was not passed in, we go ahead and add it to the dictionary.  this will get passed back to VoiceAttack for processing on return


            if (shortIntValues.ContainsKey("someValueIWantToClear")) //looking for a condition named, 'someValueIWantToClear'
                shortIntValues["someValueIWantToClear"] = null;  //the item was found... set it to null so it will be removed from the collection on return
            else
                shortIntValues.Add("someValueIWantToClear", null);  //this is in case the value was not passed in, but you want to clear the value (if it exists in VoiceAttack's list) on return

            if (dateTimeValues.ContainsKey("someDateValueToChange"))
                dateTimeValues["someDateValueToChange"] = new DateTime(2015, 12, 31);

            //here we are adding some stuff to state
            object objValue = null;
            String strStateValue = null;
            if (state.TryGetValue("myStateValue", out objValue))  //we check to see if there is a value in state for 'myStateValue'
            {
                strStateValue = (String)objValue; //if we find something, we are going to cast it as a string and use it somewhere in here...
            }
            else
                strStateValue = "initialized";  //nothing was found in state... just set the string to, 'initialized' and keep moving...


            //hope that helps some!
        }
    }
}


//this is the second namespace.  both classes could have been in the same namespace.  this is just an example.  
namespace VATestWinampPlugin
{
    //this is a very basic plugin created very quickly (maybe 20 minutes, including the profile).  Note that this class depends on the WinampLib.cs file also located in this project.

    //also note that you'll need a running instance of winamp to actually see anything happen... pick a good song and play it while you are issuing voice commands

    public class VoiceAttackSampleWinampPlugin
    {
        public static string VA_DisplayName()
        {
            return "VoiceAttack Simple Winamp Plugin";  //a name to distinguish my plugin from others
        }

        public static Guid VA_Id()
        {
            return new Guid("{99845215-1FF8-4e36-A370-87178D01C7B4}");   //note this is a new guid for this plugin
        }

        public static string VA_DisplayInfo()
        {
            return "VoiceAttack Winamp Sample Plugin\r\n\r\nThis is just a sample to show how VoiceAttack can interact with Winamp.\r\n\r\n2014 VoiceAttack";  //this is just extended info that you might want to give to the user.  note that you should format this up properly.
        }

        public static void VA_Init1(ref Dictionary<string, object> state, ref Dictionary<string, Int16?> shortIntValues, ref Dictionary<string, string> textValues, ref Dictionary<string, int?> intValues, ref Dictionary<string, decimal?> decimalValues, ref Dictionary<string, Boolean?> booleanValues, ref Dictionary<string, DateTime?> dateTimeValues, ref Dictionary<string, object> extendedValues)
        {
            //not needing to initialize anything here this time
        }

        public static void VA_Exit1(ref Dictionary<string, object> state)
        {
            //no need to do anything on exit with this implementation
        }

        public static void VA_Invoke1(String context, ref Dictionary<string, object> state, ref Dictionary<string, Int16?> shortIntValues, ref Dictionary<string, string> textValues, ref Dictionary<string, int?> intValues, ref Dictionary<string, decimal?> decimalValues, ref Dictionary<string, Boolean?> booleanValues, ref Dictionary<string, DateTime?> dateTimeValues, ref Dictionary<string, object> extendedValues)
        {
            //i was going to convert over some old delphi code I had written that does pretty much all this is doing, but I found a winamp library in c# out on code project instead.  saved a BUNCH of time :)


            //NOTE : In this example we could have more easily set the context value, but the illustration is in showing how values can be passed thru short int (used to be called, 'conditions') and text values

            Int16? iVal;  //note that this is a nullable int16... that means that the value can be int16 or null

            if (shortIntValues.TryGetValue("vaSampleWinampCommand", out iVal))  //see if the vaSampleWinampeCommand short int value exists and has a value.  note that in the included Winamp profile, vaSampleWinampeCommand is being set to a certain value, depending on the command.  if vaSampleWinampeCommand is set to 1, this means, 'play'.
            {                                                                   //2 means, 'pause', 3 means, 'stop' and so on.
                switch (iVal) //based on what is passed in, we just call out to the winamp library to make things happen.
                {
                    case 1:
                        WinampFrontEndLib.WinampLib.Play();
                        break;
                    case 2:
                        WinampFrontEndLib.WinampLib.Pause();
                        break;
                    case 3:
                        WinampFrontEndLib.WinampLib.Stop();
                        break;
                    case 4:
                        WinampFrontEndLib.WinampLib.VolumeUp();
                        break;
                    case 5:
                        WinampFrontEndLib.WinampLib.VolumeDown();
                        break;
                    case 6:  //set the volume to a value from 0 to 255
                        Int16? iVolume;
                        if (shortIntValues.TryGetValue("vaSampleWinampVolume", out iVolume))
                        {
                            if (iVolume.HasValue)
                                WinampFrontEndLib.WinampLib.SetVolume(iVolume.Value);  //iVolume can be null at this point
                        }
                        break;
                    case 7:
                        WinampFrontEndLib.WinampLib.NextTrack();
                        break;
                    case 8:
                        WinampFrontEndLib.WinampLib.PrevTrack();
                        break;
                    case 9:  //this gets the the current song title and sends it back to VA in a text value.  in the included winamp profile, you will notice that the title that is sent back is read using text-to-speech
                        String strTitle = WinampFrontEndLib.WinampLib.GetCurrentSongTitle();
                        if (textValues.ContainsKey("vaSampleWinampTextTitle"))
                            textValues["vaSampleWinampTextTitle"] = strTitle;
                        else
                            textValues.Add("vaSampleWinampTextTitle", strTitle);
                        break;
                    default:  //the catch-all for this is to just bail out (undefined or null or whatever)
                        break;
                }
            }
        }
    }
}


